package edu.gmu.trimdroid.metadata;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class ActivityMetaData {

	private String activityName;	
	private String packageName;
    private String layout;
	private boolean hasListView = false;

	private List<String> inputBoxes = new ArrayList<String>();
	private List<String> buttons  = new ArrayList<String>();

	// maps text box name to index on screen
	private Map<String,String> inputBoxToIndex = new HashMap<String, String>();

	// maps buttonID to buttonLabel
	private Map<String,String> buttonIdToLabel = new HashMap<String, String>();

	// maps radioGroupID to radioGroupLabel
	private Map<String,RadioGroupMetaData> radioGroupIdToMetaData = new HashMap<String, RadioGroupMetaData>();

	private List<RadioGroupMetaData> radioGroups = new ArrayList<RadioGroupMetaData>();
	private String layoutPath;

	public ActivityMetaData(String activityName, String packageName, String layout) {
		this.activityName = activityName;
		this.packageName = packageName;
		this.layout = layout;
	}

	public List<String> getInputBoxes() {
		return inputBoxes;
	}

	public List<String> getButtons() {
		return buttons;
	}

	public RadioGroupMetaData getRadioGroupById(String id) {
		return radioGroupIdToMetaData.get(id);
	}

	public List<RadioGroupMetaData> getRadioGroups() {
		return radioGroups;
	}

	public String getPackageName() {
		return packageName;
	}

	public String getLayoutName() {
        return this.layout;
	}

	public boolean loadFrom(String basePath, String xmlStringsFile)
	{
		//TODO: check to see if xmlLayoutFile exist
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

		try 
		{
			DocumentBuilder db = dbf.newDocumentBuilder();

            layoutPath = "/res/layout/" + this.layout + ".xml";

            if(this.layout == null || this.layout.equals(""))
                return false;

			String xmlLayoutFile = basePath+ "/res/layout/" + this.layout + ".xml";
			File f = new File(xmlLayoutFile);

			Document dom;
			if(!f.exists()) {			

				// if activity has no layout file
				if(getLayoutName() == null || getLayoutName().trim().equals("")) {
					System.out.println( this.activityName + " has no Layout file");
					return false;
				}
				File file = new File(basePath + "/res/xml/" + activityName.toLowerCase() + ".xml");
				if (file.exists()) {
					layoutPath = "/res/xml/" + activityName.toLowerCase() + ".xml";
					xmlLayoutFile = basePath + layoutPath;
				} else {
					file = new File(basePath + "/res/xml/" + this.layout + ".xml");
					if (file.exists()) {
						layoutPath = "/res/xml/" + this.layout + ".xml";
						xmlLayoutFile = basePath + layoutPath;
					}
				}
				dom = db.parse(xmlLayoutFile);
			}
			else
				dom = db.parse(xmlLayoutFile);

			Element root = dom.getDocumentElement();
			NodeList nl = root.getElementsByTagName("EditText");
			if(nl != null && nl.getLength() > 0) 
			{
				for(int i = 0 ; i < nl.getLength();i++) 
				{
					Element el = (Element)nl.item(i);
					//					String inputType = el.getAttribute("android:inputType");
					String boxId = el.getAttribute("android:id").replace("@+id/", "").replace("@id/", "");
					inputBoxToIndex.put(boxId, ""+i);
					getInputBoxes().add(boxId);
				}
			}	
			
			NodeList listView = root.getElementsByTagName("ListView");
			if(listView != null && listView.getLength() > 0) {
				hasListView = true;
			}

			DocumentBuilderFactory dbfStrings = DocumentBuilderFactory.newInstance();
			DocumentBuilder dbStrings = dbfStrings.newDocumentBuilder();
			Document domString = dbStrings.parse(xmlStringsFile);		

			nl = root.getElementsByTagName("Button");
			if(nl != null && nl.getLength() > 0) 
			{
				for(int i = 0 ; i < nl.getLength();i++) 
				{
					Element el = (Element)nl.item(i);
					String buttonLabel = el.getAttribute("android:text");
					System.out.println(buttonLabel);

					if(buttonLabel.toLowerCase().contains("@string")) {
						buttonLabel = buttonLabel.substring(8);
					}


					String stringValue = this.getLabelText(domString.getDocumentElement(), buttonLabel); 
					getButtons().add(stringValue);
					String buttonId = el.getAttribute("android:id").replace("@+id/", "").replace("@id/", "");
					buttonIdToLabel.put(buttonId,stringValue);
				}
			}				

			nl = root.getElementsByTagName("RadioGroup");
			if(nl != null && nl.getLength() > 0) 
			{
				for(int i = 0 ; i < nl.getLength();i++) 
				{
					Element el = (Element)nl.item(i);
					String radioGroupId = el.getAttribute("android:id").replace("@+id/", "").replace("@id/", "");
					
					RadioGroupMetaData radioGroup = new RadioGroupMetaData(radioGroupId);

					NodeList radioButtons = root.getElementsByTagName("RadioButton");
					if(radioButtons != null && radioButtons.getLength() > 0)  {

						for(int j = 0 ; j < radioButtons.getLength();j++) {

							Element element = (Element)radioButtons.item(j);
							String radioButtonLabel = element.getAttribute("android:text");
							System.out.println(radioButtonLabel);

							if(radioButtonLabel.toLowerCase().contains("@string")) {
								radioButtonLabel = radioButtonLabel.substring(8);
							}
							String stringValue = this.getLabelText(domString.getDocumentElement(), radioButtonLabel); 
							getButtons().add(stringValue);
							String radioButtonId = element.getAttribute("android:id").replace("@+id/", "").replace("@id/", "");
							radioGroup.addRadioButtonsID(radioButtonId);

							radioGroup.addRadioButtonsLabel(stringValue);
						}
					}



					radioGroupIdToMetaData.put(radioGroupId,radioGroup);
					radioGroups.add(radioGroup);

				}
			}

			return true;

		}catch(ParserConfigurationException pce) {
			pce.printStackTrace();
		}catch(SAXException se) {
			se.printStackTrace();
		}catch(IOException ioe) {
			ioe.printStackTrace();
		}

		return false;
	}

	private String getLabelText(Element root, String labelName)
	{
		NodeList nl = root.getElementsByTagName("string");
		int count = nl.getLength();
		for (int i=0; i < count; i++)
		{
			Element el = (Element)nl.item(i);
			if (el.getAttribute("name").equals(labelName))
				return el.getFirstChild().getNodeValue();
		}
		return null;
	}

	public String getLayoutPath() {
		return layoutPath;
	}

	
	public boolean hasListView () {
		return this.hasListView;
	}
}
