package edu.gmu.trimdroid.metadata;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import edu.gmu.trimdroid.soot.utils.Const;
import edu.gmu.trimdroid.soot.utils.Util;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class InterfaceModel
{
//	private String mApplicationName;
	private Map<String,ActivityMetaData> activityList = new HashMap<String,ActivityMetaData>(); //list of activities
    
	// a map of layout files to activities
	private Map<String,String> layoutToActivity = new HashMap<String,String>();

	public InterfaceModel(String basePath) {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		
		try
		{

			String manifestPath = basePath + "/AndroidManifest.xml";
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document dom = db.parse(manifestPath);
			Element root = dom.getDocumentElement();
            Element manifest = dom.getDocumentElement();

            String nameSpace = manifest.getAttribute("package");

            NodeList activities = manifest.getElementsByTagName("activity");

			if (activities != null)
			{
				for(int i = 0 ; i < activities.getLength();i++)
				{
					Node n = activities.item(i);

                    Element activity = (Element) activities.item(i);
                    String activityName = activity.getAttribute(Const.MANIFEST_NAME_ATTR);
                    String packageName = nameSpace;
                    if (!activityName.startsWith(".") && !packageName.endsWith("."))
                        nameSpace += ".";
                    String fullName = (activityName.startsWith(nameSpace) ? "" : nameSpace) + activityName;

					if(fullName.split("\\.")[0] != null && fullName.split("\\.")[0].length() > 0) {
						int numberOfDots = fullName.split("\\.").length;
						activityName = fullName.split("\\.")[numberOfDots-1];
                        packageName = fullName.substring(0,fullName.length() - activityName.length() - 1);
					}

					String xmlStringsFile = basePath + "/res/values/strings.xml";
					String layout = Util.extractLayout(fullName);
					if (layout == null || layout.equals("")) {
						File file = new File(basePath + "/res/xml/" + activityName.toLowerCase() + ".xml");
						if (file.exists()) {
							layout = activityName.toLowerCase();
// 							System.out.println("kir <<< this is your Easter Egg");
						}
					}
					layout = layout.replaceAll(".xml", "");

                    addActivity(activityName, packageName, layout, basePath , xmlStringsFile);

				}

			}

		}catch(ParserConfigurationException pce) {
			pce.printStackTrace();
		}catch(SAXException se) {
			se.printStackTrace();
		}catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}

	public void addActivity(String activityName, String packageName, String layout, String basePath,
			String xmlStringsFile)
	{
		ActivityMetaData activityMetaData = new ActivityMetaData(activityName, packageName, layout);


		if (activityName != null && !activityName.trim().equals(""))
		{
			if (activityMetaData.loadFrom(basePath, xmlStringsFile)) {
				activityList.put(activityName,activityMetaData);
				layoutToActivity.put(activityMetaData.getLayoutName().trim(), activityName);
			}
		}
	}

	public String getActivityFromLayout(String layoutName) {
		return layoutToActivity.get(layoutName);
	}


	public Map<String,ActivityMetaData> getActivities(){
		return activityList;
	}

}
