package edu.gmu.trimdroid.metadata;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import edu.gmu.trimdroid.AbstractXmlParser;
import edu.gmu.trimdroid.metadata.ui.CheckBox;
import edu.gmu.trimdroid.metadata.ui.CheckBoxPreference;
import edu.gmu.trimdroid.metadata.ui.DatePicker;
import edu.gmu.trimdroid.metadata.ui.EditText;
import edu.gmu.trimdroid.metadata.ui.EditTextPreference;
import edu.gmu.trimdroid.metadata.ui.ListPreference;
import edu.gmu.trimdroid.metadata.ui.NumberPicker;
import edu.gmu.trimdroid.metadata.ui.RadioGroup;
import edu.gmu.trimdroid.metadata.ui.SeekBar;
import edu.gmu.trimdroid.metadata.ui.Spinner;

public class LayoutMetaData extends AbstractXmlParser{

	private List<EditText> editTexts = new ArrayList<EditText>();
	private List<EditTextPreference> editTextPrefs = new ArrayList<EditTextPreference>();
	private List<Spinner> spinners = new ArrayList<Spinner>();
	private List<DatePicker> datePickers = new ArrayList<DatePicker>();
	private List<CheckBox> checkBoxes = new ArrayList<CheckBox>();
	private List<CheckBoxPreference> checkBoxePrefs = new ArrayList<CheckBoxPreference>();
	private List<ListPreference> listPrefs = new ArrayList<ListPreference>();
	private List<RadioGroup> radioGroups = new ArrayList<RadioGroup>();
	private List<SeekBar> seekBars = new ArrayList<>();
	private List<NumberPicker> numberPickers = new ArrayList<>();
	
	private List<String> widgetsList = new ArrayList<String>();

	private Map<String,String> idToType = new HashMap<String,String>();

	public LayoutMetaData(String inputXMLPath, String stringsXml){

		Document document = prepareXML(inputXMLPath);

		if (document!=null) {

			NodeList editTexts = document.getElementsByTagName("EditText");

			// TextEdit
			if(editTexts != null && editTexts.getLength() > 0) {
				for(int j = 0 ; j < editTexts.getLength();j++) {

					Element el = (Element)editTexts.item(j);
					if (el.getAttribute("android:id") == null)
						continue;


					String id = el.getAttribute("android:id");
					id = id.split("/")[1].trim();

					String inputType = el.getAttribute("android:inputType").trim();



					if ( inputType != null && inputType.trim() != "") {
						if(inputType.equals("textPassword") || inputType.equals("text"))
							inputType = "string";						
						if(inputType.equals("number") || inputType.equals("numberSigned")) 
							inputType = "numberDecimal";	
						else
							inputType = "string";
					}
					else {
						inputType = el.getAttribute("android:numeric").trim();
						if (inputType.equals("decimal")) {
							inputType = "numberDecimal";
						} else if (inputType.equals("integer")) {
							inputType = "integer";
						} else
							inputType = "string";
					}

					EditText editText = new EditText(id,inputType);
					idToType.put(id, EditText.TYPE);
					this.editTexts.add(editText);
					this.widgetsList.add(id);
				}
			} 

			NodeList editTextPrefs = document.getElementsByTagName("EditTextPreference");

			// TextEditPrefrence
			if(editTextPrefs != null && editTextPrefs.getLength() > 0) {
				for(int j = 0 ; j < editTextPrefs.getLength();j++) {

					Element el = (Element)editTextPrefs.item(j);
					if (el.getAttribute("android:id") == null)
						continue;

					String id = "ETP" + j;

					if (el.getAttribute("android:title") == null || !el.getAttribute("android:title").contains("/"))
						continue;
					
					String title = el.getAttribute("android:title").split("/")[1].trim();

					String inputType = el.getAttribute("android:inputType").trim();

					if ( inputType != null && inputType.trim() != "") {
						if(inputType.equals("textPassword"))
							inputType = "string";						
						if(inputType.equals("number") || inputType.equals("numberSigned") || inputType.equals("phone")) 
							inputType = "numberDecimal";													
					}
					else {
						inputType = el.getAttribute("android:numeric").trim();
						if (inputType.equals("decimal")) {
							inputType = "numberDecimal";
						} else if (inputType.equals("integer")) {
							inputType = "integer";
						} else
							inputType = "string";
					}

					EditTextPreference editTextPref = new EditTextPreference(id, title, inputType);
					idToType.put(id, EditTextPreference.TYPE);
					this.editTextPrefs.add(editTextPref);
					this.widgetsList.add(id);
				}
			} 

			// Radio Buttons
			NodeList radioGroups = document.getElementsByTagName("RadioGroup");

			if(radioGroups != null && radioGroups.getLength() > 0) {
				for(int j = 0 ; j < radioGroups.getLength();j++) {

					int radioButtonCount = 0 ;
					Element el = (Element)radioGroups.item(j);
					if (el.getAttribute("android:id") == null)
						continue;

					NodeList radioButtons = el.getElementsByTagName("RadioButton");

					if(radioButtons != null && radioButtons.getLength() > 0) {
						radioButtonCount = radioButtons.getLength();
					} else
						continue;

					String id = el.getAttribute("android:id");
					id = id.split("/")[1].trim();

					RadioGroup radioGroup = new RadioGroup(id,radioButtonCount);
					idToType.put(id, RadioGroup.TYPE);
					this.radioGroups.add(radioGroup);
					this.widgetsList.add(id);
				}
			} 

			// DatePicker
			NodeList datePickers = document.getElementsByTagName("DatePicker");

			if(datePickers != null && datePickers.getLength() > 0) {
				for(int j = 0 ; j < datePickers.getLength();j++) {

					Element el = (Element)datePickers.item(j);
					if (el.getAttribute("android:id") == null)
						continue;

					String id = el.getAttribute("android:id");
					id = id.split("/")[1].trim();

					DatePicker datePicker = new DatePicker(id);
					idToType.put(id, DatePicker.TYPE);
					this.datePickers.add(datePicker);
					this.widgetsList.add(id);
				}
			} 

			// CheckBox
			NodeList checkBoxes = document.getElementsByTagName("CheckBox");

			if(checkBoxes != null && checkBoxes.getLength() > 0) {
				for(int j = 0 ; j < checkBoxes.getLength();j++) {

					Element el = (Element)checkBoxes.item(j);
					if (el.getAttribute("android:id") == null)
						continue;

					String id = el.getAttribute("android:id");
					id = id.split("/")[1].trim();

					CheckBox checkBox = new CheckBox(id);
					idToType.put(id, CheckBox.TYPE);
					this.checkBoxes.add(checkBox);
					this.widgetsList.add(id);
				}
			} 

			// SeekBars
			NodeList seekbars = document.getElementsByTagName("net.mandaria.preferences.SeekBarPreference");

			if(seekbars != null && seekbars.getLength() > 0) {
				for(int j = 0 ; j < seekbars.getLength();j++) {

					Element el = (Element)seekbars.item(j);

					String id = "SB" + j;

					String title = el.getAttribute("android:title");
					if (title.startsWith("@string/"))
						title = title.split("/")[1].trim();
					title = title.replaceAll("\\s+", "___");

					SeekBar seekBar = new SeekBar(id, title);
					idToType.put(id, SeekBar.TYPE);
					this.seekBars.add(seekBar);
					this.widgetsList.add(id);
				}
			}

			// numberPickers
			NodeList numberPickers = document.getElementsByTagName("net.mandaria.preferences.DecimalPreference");

			if(numberPickers != null && numberPickers.getLength() > 0) {
				for(int j = 0 ; j < numberPickers.getLength();j++) {

					Element el = (Element)numberPickers.item(j);

					String id = "NP" + j;
					
					String title = el.getAttribute("android:title");
					if (title.startsWith("@string/"))
						title = title.split("/")[1].trim();
					title = title.replaceAll("\\s+", "___");

					NumberPicker numberPicker = new NumberPicker(id, title);
					idToType.put(id, NumberPicker.TYPE);
					this.numberPickers.add(numberPicker);
					this.widgetsList.add(id);
				}
			}


			// numberPickers
			numberPickers = document.getElementsByTagName("net.mandaria.preferences.NumberPickerPreference");

			if(numberPickers != null && numberPickers.getLength() > 0) {
				for(int j = 0 ; j < numberPickers.getLength();j++) {

					Element el = (Element)numberPickers.item(j);

					String id = "NP" + j + this.numberPickers.size();
					
					String title = el.getAttribute("android:title");
					if (title.startsWith("@string/"))
						title = title.split("/")[1].trim();
					title = title.replaceAll("\\s+", "___");

					NumberPicker numberPicker = new NumberPicker(id, title);
					idToType.put(id, NumberPicker.TYPE);
					this.numberPickers.add(numberPicker);
					this.widgetsList.add(id);
				}
			}
			
			// CheckBoxPreference
			NodeList checkBoxePrefs = document.getElementsByTagName("CheckBoxPreference");

			if(checkBoxePrefs != null && checkBoxePrefs.getLength() > 0) {
				for(int j = 0 ; j < checkBoxePrefs.getLength();j++) {

					Element el = (Element)checkBoxePrefs.item(j);

					if (el.getAttribute("android:title") == null || el.getAttribute("android:title").isEmpty())
						break;

					String title = el.getAttribute("android:title").split("/")[1].trim();

					String id = "CBP" + j;

					CheckBoxPreference checkBox = new CheckBoxPreference(id, title);
					idToType.put(id, CheckBoxPreference.TYPE);
					this.checkBoxePrefs.add(checkBox);
					this.widgetsList.add(id);
				}
			} 

			// Spinner
			NodeList spinners = document.getElementsByTagName("Spinner");

			if(spinners != null && spinners.getLength() > 0) {
				for(int j = 0 ; j < spinners.getLength();j++) {

					int spinnerItemsCount = 0;
					Element el = (Element)spinners.item(j);
					if (el.getAttribute("android:id") == null)
						continue;

					if (el.getAttribute("android:entries") == null)
						continue;
					
					//TODO: fix this
					if (!el.getAttribute("android:entries").contains("/"))
						continue;
					
					String entryArrayName = el.getAttribute("android:entries").split("/")[1].trim();


					// open strings.xml
					Document strings = prepareXML(stringsXml);
					NodeList stringArrays = strings.getElementsByTagName("string-array");

					if(stringArrays != null && stringArrays.getLength() > 0) {
						for(int k = 0 ; k < stringArrays.getLength(); k++) {
							Element element = (Element)stringArrays.item(k);
							String arrayName = element.getAttribute("name");
							if(entryArrayName.equals(arrayName)) {
								NodeList items = element.getElementsByTagName("item");
								if(items != null && items.getLength() > 0) {
									spinnerItemsCount = items.getLength();
								}
								else {
									System.out.println("THIS SHOULD NOT HAPPEN: NO ITEMS");
								}
							}

						}
					} else {
						// open arrays.xml
						String temp = stringsXml.replace("strings", "arrays");
						Document arrays = prepareXML(temp);
						stringArrays = arrays.getElementsByTagName("string-array");

						if(stringArrays != null && stringArrays.getLength() > 0) {
							for(int k = 0 ; k < stringArrays.getLength(); k++) {
								Element element = (Element)stringArrays.item(k);
								String arrayName = element.getAttribute("name");
								if(entryArrayName.equals(arrayName)) {
									NodeList items = element.getElementsByTagName("item");
									if(items != null && items.getLength() > 0) {
										spinnerItemsCount = items.getLength();
									}
									else {
										System.out.println("THIS SHOULD NOT HAPPEN: NO ITEMS");
									}
								}

							}
						} else 
							System.out.println("THIS SHOULD NOT HAPPEN: EMPTY ARRAY");
					}

					String id = el.getAttribute("android:id");
					id = id.split("/")[1].trim();

					Spinner spinner = new Spinner(id, spinnerItemsCount);
					idToType.put(id,Spinner.TYPE);
					this.spinners.add(spinner);
					this.widgetsList.add(id);
				}
			} 
			
			// List Preference
			NodeList listPrefs = document.getElementsByTagName("ListPreference");
			
			if(listPrefs != null && listPrefs.getLength() > 0) {
				for(int j = 0 ; j < listPrefs.getLength();j++) {

					int listItemsCount = 0;
					Element el = (Element)listPrefs.item(j);
					if (el.getAttribute("android:id") == null)
						continue;

					if (el.getAttribute("android:entries") == null)
						continue;

					if (!el.getAttribute("android:entries").contains("/"))
						continue;

					String entryArrayName = el.getAttribute("android:entries").split("/")[1].trim();


					// open strings.xml
					Document strings = prepareXML(stringsXml);
					NodeList stringArrays = strings.getElementsByTagName("string-array");

					if(stringArrays != null && stringArrays.getLength() > 0) {
						for(int k = 0 ; k < stringArrays.getLength(); k++) {
							Element element = (Element)stringArrays.item(k);
							String arrayName = element.getAttribute("name");
							if(entryArrayName.equals(arrayName)) {
								NodeList items = element.getElementsByTagName("item");
								if(items != null && items.getLength() > 0) {
									listItemsCount = items.getLength();
								}
								else {
									System.out.println("THIS SHOULD NOT HAPPEN: NO ITEMS");
								}
							}

						}
					} else {
						// open arrays.xml
						String temp = stringsXml.replace("strings", "arrays");
						Document arrays = prepareXML(temp);
						stringArrays = arrays.getElementsByTagName("string-array");

						if(stringArrays != null && stringArrays.getLength() > 0) {
							for(int k = 0 ; k < stringArrays.getLength(); k++) {
								Element element = (Element)stringArrays.item(k);
								String arrayName = element.getAttribute("name");
								if(entryArrayName.equals(arrayName)) {
									NodeList items = element.getElementsByTagName("item");
									if(items != null && items.getLength() > 0) {
										listItemsCount = items.getLength();
									}
									else {
										System.out.println("THIS SHOULD NOT HAPPEN: NO ITEMS");
									}
								}

							}
						} else 
							System.out.println("THIS SHOULD NOT HAPPEN: EMPTY ARRAY");
					}


					String id = "LP" + j;

					if(id.contains("\\/"))
						id = id.split("/")[1].trim();

					String title = el.getAttribute("android:title");
					if (title.startsWith("@string/"))
						title = title.split("/")[1].trim();
					title = title.replaceAll("\\s+", "___");
						
					ListPreference listPref = new ListPreference(id, listItemsCount,title);
					idToType.put(id,ListPreference.TYPE);
					this.listPrefs.add(listPref);
					this.widgetsList.add(id);
				}
			} 
		}
	}

	public List<EditText> getEditTexts() {
		return editTexts;
	}

	public List<Spinner> getSpinners() {
		return spinners;
	}

	public List<DatePicker> getDatePickers() {
		return datePickers;
	}

	public List<CheckBox> getCheckBoxes() {
		return checkBoxes;
	}

	public List<RadioGroup> getRadioGroups() {
		return radioGroups;
	}

	public List<EditTextPreference> getEditTextPrefs() {
		return editTextPrefs;
	}

	public List<CheckBoxPreference> getCheckBoxePrefs() {
		return checkBoxePrefs;
	}

	public String getIdToType(String id) {
		return idToType.get(id);
	}

	public List<String> getWidgetsIdList() {
		return widgetsList;
	}				

	public List<ListPreference> getListPrefs() {
		return listPrefs;
	}
	
	public List<SeekBar> getSeekBars() {
		return seekBars;
	}

	public List<NumberPicker> getNumberPickers() {
		return numberPickers;
	}

	public boolean contains(String widgetID) {

		if(widgetsList.contains(widgetID))
			return true;
		else 
			return false;	
	}
}