package edu.gmu.trimdroid.soot.callgraph;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import soot.Body;
import soot.PatchingChain;
import soot.SootClass;
import soot.SootMethod;
import soot.Unit;
import soot.Value;
import soot.jimple.internal.JLookupSwitchStmt;
import soot.toolkits.graph.Block;
import soot.toolkits.graph.CompleteBlockGraph;
import edu.gmu.trimdroid.soot.callgraph.CGEdge.EdgeType;
import edu.gmu.trimdroid.soot.utils.EventHandlerExtractor;
import edu.gmu.trimdroid.soot.utils.Util;

public class GUINodeExtractor implements NodeExtractor {

	@Override
	public List<CGNode> extractNode(Unit unit, PatchingChain<Unit> units, SootMethod thisMethod, CGNode fromNode, SootClass rClazz) {
		Object[] widgetHandlerAndId = EventHandlerExtractor.findWidgetHandlerAndId(units, unit);
		SootMethod listener = (SootMethod) widgetHandlerAndId[0];
		Value widgetId = (Value) widgetHandlerAndId[1];
		CGNode connectNodes = connectNodes(listener, widgetId, fromNode, rClazz);
		return Arrays.asList(new CGNode[] { connectNodes });
	}

	private static CGNode connectNodes(SootMethod listener, Object widgetId, CGNode fromNode, SootClass rClazz) {
		if (listener == null)
			return null;
		CGNode toNode = new CGNode(listener);
		return connectNodes(toNode, widgetId, fromNode, rClazz);
	}

	private static CGNode connectNodes(CGNode toNode, Object widgetId, CGNode fromNode, SootClass rClazz) {
		if (toNode == null)
			return null;
//		// TODO: this change was mad on Jan 28 to fix Tipster
		if (toNode.getMethodName().contains("onClick") && fromNode.getMethodName().contains("onClick") || widgetId == null)
			return null;
		String widgetIdStr = Util.integerId2StringId(widgetId, rClazz);
		CGEdge edge = new CGEdge(widgetIdStr, EdgeType.GUI);
		CGNode.connect(fromNode, toNode, edge, true);
		return toNode;
	}

	public static List<CGNode> extractMenuNodes(SootMethod method, CGNode fromNode, SootClass rClazz) {
		List<CGNode> nodes = new ArrayList<>();
		Body body = method.getActiveBody();
		PatchingChain<Unit> units = body.getUnits();
		for (Unit unit : units) {
			if (unit instanceof JLookupSwitchStmt) {
				JLookupSwitchStmt menuSwitch = (JLookupSwitchStmt) unit;
				for (int index = 0; index < menuSwitch.getTargetCount(); index++) {
					int lookupValue = menuSwitch.getLookupValue(index);
					Unit target = menuSwitch.getTarget(index);
					Integer widgetId = new Integer(lookupValue);
					List<CGNode> processedNodes = processCaseBlocks(body, target, widgetId, fromNode, rClazz, units);
					nodes.addAll(processedNodes);
					if(processedNodes.isEmpty())
						nodes.add(connectNodes(new CGNode("something."+fromNode.getShortLabel().split("\\(")[0].trim() + ": Self Loop()"), widgetId, fromNode, rClazz));
				}
			}
		}
		return nodes;
	}

	private static List<CGNode> processCaseBlocks(Body b, Unit target, Integer widgetId, CGNode fromNode, SootClass rClazz, PatchingChain<Unit> units) {
		List<CGNode> nodes = new ArrayList<>();
		CompleteBlockGraph blockGraph = new CompleteBlockGraph(b);
		List<Block> blocks = blockGraph.getBlocks();
		for (Block block : blocks) {
			if (block.getHead() != null && block.getHead().equals(target)) {
				for (Unit unit : block) {
					CGNode simpleCallNode = connectNodes(SimpleNodeExtractor.extractSimpleCallNode(unit), widgetId, fromNode, rClazz);
					if (simpleCallNode != null)
						nodes.add(simpleCallNode);
					List<CGNode> ipcNodes = IPCNodeExtractor.extractIPCNode(unit, b.getMethod(), units);
					for (CGNode ipcNode : ipcNodes) {
						nodes.add(connectNodes(ipcNode, widgetId, fromNode, rClazz));
					}
				}
			}
		}
		return nodes;
	}

}
