package edu.gmu.trimdroid.soot.depextract;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import soot.*;
import soot.toolkits.graph.CompleteBlockGraph;
import edu.gmu.trimdroid.soot.utils.Util;

public abstract class AbstractDependencyExtractor {

	PatchingChain<Unit> units;
	CompleteBlockGraph blockGraph;
    SootClass rClazz;

    public void preAnalysis(PatchingChain<Unit> unit, CompleteBlockGraph graph) {
		this.units = unit;
		this.blockGraph = graph;
        this.rClazz = Util.findRClass("id");
	}

	public abstract void doDependencyAnalysis(Body b);
	
	protected abstract Map<String, HashSet<String>> getConditionsMap();
	
	protected abstract List<List<String>> getDependencies();

	public Map<String, HashSet<String>> getInputClassesMap(){
		Map<String, HashSet<String>> result = new HashMap<>();
        Iterator<Entry<String, HashSet<String>>> it = getConditionsMap().entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry pair = (Map.Entry)it.next();
            
            result.put(Util.integerId2StringId(pair.getKey(),rClazz), (HashSet<String>) pair.getValue());
            it.remove(); 
        }
        return result;
	}
	
    public List<List<String>> getDependencyIDs() {
        List<List<String>> result = new ArrayList<>();
        for(List<String> l:getDependencies()) {
            List<String> tmp = new ArrayList<>();

            for(String s:l) {
                tmp.add(Util.integerId2StringId(s,rClazz));
            }
            result.add(tmp);
        }
        return result;
    }

	public abstract String getDependencyType();

	protected Unit findClosestDefPointBackward(Unit usedPoint, SootFieldRef fieldRef) {
		return Util.findClosestDefPointBackward(usedPoint, fieldRef, units);
	}

	protected Unit findClosestDefPointBackward(Unit usedPoint, Value var) {
		return Util.findClosestDefPointBackward(usedPoint, var, units);
	}

	protected Unit findClosestUsePointForward(Unit defPoint, SootFieldRef fieldRef) {
		return Util.findClosestUsePointForward(defPoint, fieldRef, units);
	}

	protected Unit findClosestUsePointForward(Unit defPoint, Value var) {
		return Util.findClosestUsePointForward(defPoint, var, units);
	}
}
