package edu.gmu.trimdroid.soot.depextract;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import com.sun.java.swing.plaf.windows.resources.windows;

import soot.Body;
import soot.BodyTransformer;
import soot.PackManager;
import soot.Transform;
import soot.options.Options;
import soot.toolkits.graph.CompleteBlockGraph;

public class DependencyExtractor extends BodyTransformer {

	private AbstractDependencyExtractor[] extractors;
	private List<List<String>> conditionalDependecies = new ArrayList<>();
	private List<List<String>> handlerDependecies = new ArrayList<>();
	private List<List<String>> intentDependecies = new ArrayList<>();
	private Map<String, HashSet<String>> conditionsMap = new HashMap<>();

	public DependencyExtractor(String apkFilePath) {
		extractors = new AbstractDependencyExtractor[] { new WidgetDependencyExtractor(), new HandlerDependencyExtractor() };
		PackManager.v().getPack("jtp").add(new Transform("jtp.androidTransformer", this));
		applySootOptions();
		soot.Main.main(new String[] { "-process-dir", apkFilePath });
		buildResults();
	}

	public List<List<String>> getConditionalDependecies() {
		return conditionalDependecies;
	}

	public List<List<String>> getHandlerDependecies() {
		return handlerDependecies;
	}

	public List<List<String>> getIntentDependecies() {
		return intentDependecies;
	}

	private void buildResults() {

		for (AbstractDependencyExtractor extractor : extractors) {
			System.out.println("Dependency Type :" + extractor.getDependencyType());
			if (extractor.getDependencyType().contains("Conditional")) {
				conditionsMap.putAll(extractor.getInputClassesMap());

				for (List<String> list : extractor.getDependencyIDs()) {
					conditionalDependecies.add(list);
				}
			}

			if (extractor.getDependencyType().contains("Handler")) {
				for (List<String> list : extractor.getDependencyIDs()) {
					handlerDependecies.add(list);
				}
			}

			if (extractor.getDependencyType().contains("Intent")) {
				for (List<String> list : extractor.getDependencyIDs()) {
					intentDependecies.add(list);
				}
			}

		}
	}

	// takes action widget ID as input and returns all dependent
	public List<String> findHandlerDependencies(String id) {
		List<String> results = new ArrayList<String>();
		for (List<String> l : handlerDependecies) {
			if (l.contains(id)) {
				for (String s : l)
					if (!results.contains(s.trim()) && !s.trim().equals(id))
						results.add(s.trim());
			}
		}
		return results;
	}

	// takes data widget ID as input and returns all dependent widgets
	public List<List<String>> findConditionalDependencies4Handler(List<String> handlerDependencies) {
		List<List<String>> results = new ArrayList<List<String>>();
		for (String widget : handlerDependencies) {
			// flag is true if widget belongs to any widget dependency set
			boolean flag = false;
			for (List<String> l : conditionalDependecies) {
				if (l.contains(widget)) {
					if (!results.contains(l)) {
						results.add(l);
						flag = true;
					}
				}
			}
			// if widget does not belong to any set add it by itself
			if (!flag) {
				List<String> temp = new ArrayList<String>();
				temp.add(widget);
				results.add(temp);
			}
		}
		return results;
	}

	private void applySootOptions() {
		Options.v().set_src_prec(Options.src_prec_apk);
		Options.v().set_output_format(Options.output_format_none);
		// Options.v().set_output_format(Options.output_format_jimple);
		Options.v().set_no_bodies_for_excluded(true);
		Options.v().set_show_exception_dests(false);
		Options.v().set_print_tags_in_output(false);
		Options.v().set_verbose(false);
		Options.v().set_android_jars(System.getenv("ANDROID_HOME") + "/platforms");
	}

	@Override
	protected void internalTransform(Body b, String phaseName, Map<String, String> options) {
		for (AbstractDependencyExtractor extractor : extractors) {
			extractor.preAnalysis(b.getUnits(), new CompleteBlockGraph(b));
			extractor.doDependencyAnalysis(b);
		}
	}

	public Map<String, HashSet<String>> getInputClassesMap() {
		return conditionsMap;
	}
}
