package edu.gmu.trimdroid.soot.depextract;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

//import com.microsoft.z3.BoolExpr;

//import edu.gmu.trimdroid.symengine.ConditionSolver;
import soot.Body;
import soot.SootFieldRef;
import soot.Unit;
import soot.Value;
import soot.ValueBox;
import soot.jimple.InvokeExpr;
import soot.jimple.internal.JIfStmt;
import soot.jimple.internal.JInstanceFieldRef;
import soot.jimple.internal.JimpleLocal;
import soot.toolkits.graph.Block;

public class WidgetDependencyExtractor extends AbstractDependencyExtractor {

	Map<String, List<String>> widgetFieldMap;
	Map<String, HashSet<String>> conditionsMap = new HashMap<>();
//	ConditionSolver solver;
	
	public WidgetDependencyExtractor() {
		super();
		this.widgetFieldMap = new HashMap<String, List<String>>();
//		this.solver = new ConditionSolver();
	}

	@Override
	protected List<List<String>> getDependencies() {
		List<List<String>> result = new ArrayList<>();
		List<String> ids = new ArrayList<String>(widgetFieldMap.keySet());
		for (int i = 0; i < ids.size() - 1; i++) {
			for (int j = i + 1; j < ids.size(); j++) {
				List<String> fields_i = widgetFieldMap.get(ids.get(i));
				List<String> fields_j = widgetFieldMap.get(ids.get(j));
				for (String field : fields_i) {
					if (fields_j.contains(field))
						result.add(Arrays.asList(new String[] { ids.get(i), ids.get(j) }));
				}
			}
		}
		return result;
	}

	@Override
	public void doDependencyAnalysis(Body b) {
//		List<String> allIfStatements = new ArrayList<>();
		for (Unit unit : units) {
//			allIfStatements.add(unit.toString());
			if (unit instanceof JIfStmt) {
				Value widgetId = findWidgetId(unit);
				if (widgetId != null) {
					// Indicates that a widget used as the IF condition
					for (Block block : getIfBlocks((JIfStmt) unit)) {
						findAllDefFields(block, widgetId.toString());
					}

//					BoolExpr currExpr = solver.convertToZ3Expr(unit, b.getMethod());
//					if (currExpr != null) {
////						System.out.println("kir");
//						List<String> results = solver.findSolutionForPaths(currExpr);
//						if (conditionsMap.get(widgetId.toString()) != null) {
//							conditionsMap.get(widgetId.toString()).addAll(results);
//						} 
//						else {
//							conditionsMap.put(widgetId.toString(), new HashSet<String>(results));
//						}
//					}
				}
			}
		}
		
//		System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
//		for (String s:allIfStatements) {
//			System.out.println(s);
//		}
//		System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
	}

	private List<Block> getIfBlocks(JIfStmt ifStmt) {
		List<Block> result = new ArrayList<>();
		for (Block block : blockGraph.getBlocks()) {
			if (block.getTail().equals(ifStmt)) {
				List<Block> ifSuccsBlocks = block.getSuccs();
				boolean tmp = false;
				for (Block ifSuccsBlock : ifSuccsBlocks) {
					boolean ifBody = false;
					for (Block ifSuccsSuccBlock : ifSuccsBlock.getSuccs()) {
						for (Block myBlock : ifSuccsBlocks) {
							if (ifSuccsSuccBlock.equals(myBlock)) {
								ifBody = true;
								tmp = true;
							}
						}
					}
					if (ifBody)
						result.add(ifSuccsBlock);
				}
				if (!tmp)
					result.addAll(ifSuccsBlocks);
			}
		}
		return result;
	}

	private Value findWidgetId(Unit usedPoint) {
		// Check if the widget is looked up by Id
		for (ValueBox useBox : usedPoint.getUseBoxes()) {
			if (useBox.getValue() instanceof InvokeExpr) {
				InvokeExpr invokeExpr = (InvokeExpr) useBox.getValue();
				if (invokeExpr.getMethodRef().name().matches("^findViewById$")) {
					return invokeExpr.getArg(0);
				}
			}
		}
		// Otherwise do backward search for finding widget
		for (ValueBox useBox : usedPoint.getUseBoxes()) {
			Unit defPoint = null;
			Value usedVar = useBox.getValue();
			if (usedVar instanceof JInstanceFieldRef) {
				JInstanceFieldRef ref = ((JInstanceFieldRef) usedVar);
				defPoint = findClosestDefPointBackward(usedPoint, ref.getFieldRef());
			}
			if (usedVar instanceof JimpleLocal) {
				defPoint = findClosestDefPointBackward(usedPoint, usedVar);
			}
			if (defPoint != null) {
				Value widgetId = findWidgetId(defPoint);
				if (widgetId != null)
					return widgetId;
			}
		}
		return null;
	}


	private void findAllDefFields(Block block, String widgetId) {
		for (Unit unit : block) {
			for (ValueBox defBox : unit.getDefBoxes()) {
				if (defBox.getValue() instanceof JInstanceFieldRef) {
					SootFieldRef fieldRef = ((JInstanceFieldRef) defBox.getValue()).getFieldRef();
					List<String> fields = widgetFieldMap.get(widgetId);
					if (fields == null)
						fields = new ArrayList<>();
					fields.add(fieldRef.getSignature());
					widgetFieldMap.put(widgetId, fields);
				}
			}
		}
	}

	@Override
	public String getDependencyType() {
		return "Conditional Dependency";
	}

	public Map<String, HashSet<String>> getConditionsMap() {
		return conditionsMap;
	}
}
