package edu.gmu.trimdroid.soot.utils;

import java.util.Arrays;
import java.util.List;

import soot.PatchingChain;
import soot.RefType;
import soot.SootClass;
import soot.SootMethod;
import soot.SootMethodRef;
import soot.Type;
import soot.Unit;
import soot.Value;
import soot.ValueBox;
import soot.jimple.InvokeExpr;
import soot.jimple.internal.JAssignStmt;
import soot.jimple.internal.JVirtualInvokeExpr;

public class EventHandlerExtractor {

	public static Value findWidgetId(PatchingChain<Unit> units, Unit setterPoint, Value widget) {
		//TODO: finding widgetId is not field sensitive!
		Unit pred = null;
		while ((pred = units.getPredOf(setterPoint)) != null) {
			for (ValueBox defBox : pred.getDefBoxes()) {
				if (defBox.getValue().equals(widget)) {
					for (ValueBox useBox : pred.getUseBoxes()) {
						if (useBox.getValue() instanceof InvokeExpr) {
							InvokeExpr invokeExpr = (InvokeExpr) useBox.getValue();
							if (invokeExpr.getMethodRef().name().matches("^" + Const.METHOD_FIND_VIEW_BY_ID + "$|^" + Const.METHOD_FIND_PREFERENCE + "$")) {
								return invokeExpr.getArg(0);
							}
						} else if (pred instanceof JAssignStmt) {
							JAssignStmt stmt = (JAssignStmt) pred;
							if (stmt.leftBox.getValue().equals(widget) && useBox.getValue().getType().toString().equals(Const.TYPE_VIEW))
								widget = useBox.getValue();
						}
					}
				}
			}
			for (ValueBox useBox : pred.getUseBoxes()) {
				if (useBox.getValue() instanceof JVirtualInvokeExpr) {
					JVirtualInvokeExpr invokeExpr = (JVirtualInvokeExpr) useBox.getValue();
					if (invokeExpr.getBase().equals(widget) && invokeExpr.getMethodRef().name().matches("^" + Const.METHOD_SET_TITLE + "$")) {
						return invokeExpr.getArg(0);
					}
				}
			}
			setterPoint = pred;
		}
		return null;
	}

	public static SootMethod findListener(PatchingChain<Unit> units, Unit setterPoint, Value listener, String methodName) {
		Unit pred = null;
		while ((pred = units.getPredOf(setterPoint)) != null) {
			for (ValueBox defBox : pred.getDefBoxes()) {
				if (defBox.getValue().equals(listener)) {
					for (ValueBox useBox : pred.getUseBoxes()) {
						Type type = useBox.getValue().getType();
						if (type instanceof RefType) {
							SootClass sootClass = ((RefType) type).getSootClass();
							Iterable<SootClass> interfaces;
							if (sootClass.isInterface()) {
								interfaces = Arrays.asList(new SootClass[] { sootClass });
							} else {
								interfaces = sootClass.getInterfaces();
							}
							for (SootClass inf : interfaces) {
								String infName = inf.getName().contains("$") ? inf.getName().substring(inf.getName().indexOf("$") + 1) : inf.getName();
								if (methodName.equals("set" + infName)) {
									try {
										String listenerMethodName = String.valueOf(methodName.charAt(3)).toLowerCase()
												+ methodName.substring(4, methodName.length() - 8);
										SootMethod handlerMethod = null;
										try {
											handlerMethod = sootClass.getMethodByName(listenerMethodName);
										} catch (Exception e) {
											if (handlerMethod == null) {
												if (listenerMethodName.endsWith("Change"))
													listenerMethodName += "d";
												handlerMethod = sootClass.getMethodByName(listenerMethodName);
											}
										}
										// To address the "Change" -> "Changed"
										// issue
										return handlerMethod;
									} catch (RuntimeException e) {
									}
								}
							}
						}
					}
				}
			}
			setterPoint = pred;
		}
		return null;
	}

	public static Object[] findWidgetHandlerAndId(PatchingChain<Unit> units, Unit unit) {
		List<ValueBox> useBoxes = unit.getUseBoxes();
		for (ValueBox valueBox : useBoxes) {
			Value value = valueBox.getValue();
			if (value instanceof JVirtualInvokeExpr) {
				JVirtualInvokeExpr invokeExpr = (JVirtualInvokeExpr) value;
				SootMethodRef methodRef = invokeExpr.getMethodRef();
				String methodName = methodRef.name();
				if (methodName.matches("^setOn.*Listener$")) {
					SootMethod listener = EventHandlerExtractor.findListener(units, unit, invokeExpr.getArg(0), methodName);
					Value widgetId = EventHandlerExtractor.findWidgetId(units, unit, invokeExpr.getBase());
					return new Object[] { listener, widgetId, methodRef };
				} else if (invokeExpr.getMethod().getDeclaringClass().getName().equals(Const.TYPE_ALERT_DIALOG$BUILDER)) {
					if (methodName.matches("^set.*Button$") || methodName.equals(Const.METHOD_SET_ITEMS)) {
						SootMethod listener = EventHandlerExtractor.findListener(units, unit, invokeExpr.getArg(1), Const.METHOD_SET_ON_CLICK_LISTERNER);
						Value widgetId = null;
						if (invokeExpr.getArg(0).getType().toString().equals("int"))
							widgetId = invokeExpr.getArg(0);
						return new Object[] { listener, widgetId, methodRef };
					}
				}
			}
		}
		return new Object[] { null, null, null };
	}
}
